/*
 * tmtd - Suckless To Do list
 * Copyright (C) 2022 lemon.sh & famfo
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of  MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

use crate::{Config, Database};
use std::sync::Arc;
use tokio::sync::broadcast;
use warp::Filter;
use tera::Tera;

pub struct App {
    config: Arc<Config>,
    db: Arc<Database>,
    tera: Tera
}

impl App {
    pub fn new(config: Arc<Config>, db: Arc<Database>) -> Result<Self, tera::Error> {
        Ok(Self {
            config,
            db,
            tera: Tera::new("templates/**/*.html")?
        })
    }

    pub async fn run(self, mut cancel: broadcast::Receiver<()>) {
        let hello = warp::path!("hello" / String).map(|name| format!("Hello, {}!", name));

        let (_, server) =
            warp::serve(hello).bind_with_graceful_shutdown(self.config.listen_addr, async move {
                cancel.recv().await.unwrap();
            });
        server.await;
    }
}